/*
 * gfxoutput.c - Graphics output driver.
 *
 * Written by
 *  Andreas Boose <viceteam@t-online.de>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bmpdrv.h"
#include "color.h"
#include "gfxoutput.h"
#include "lib.h"
#include "log.h"
#include "palette.h"
#include "iffdrv.h"
#include "pcxdrv.h"
#include "ppmdrv.h"

#ifdef HAVE_PNG
#include "pngdrv.h"
#endif

#if defined(HAVE_GIF) || defined(WIN32)
#include "gifdrv.h"
#endif

#ifdef HAVE_JPEG
#include "jpegdrv.h"
#endif

#ifdef HAVE_FFMPEG
#include "ffmpegdrv.h"
#include "ffmpeglib.h"
#endif


struct gfxoutputdrv_list_s {
    struct gfxoutputdrv_s *drv;
    struct gfxoutputdrv_list_s *next;
};
typedef struct gfxoutputdrv_list_s gfxoutputdrv_list_t;

static gfxoutputdrv_list_t *gfxoutputdrv_list = NULL;
static int gfxoutputdrv_list_count = 0;
static log_t gfxoutput_log = LOG_ERR;
static gfxoutputdrv_list_t *gfxoutputdrv_list_iter = NULL;


int gfxoutput_num_drivers(void)
{
    return gfxoutputdrv_list_count;
}

gfxoutputdrv_t *gfxoutput_drivers_iter_init(void)
{
    gfxoutputdrv_list_iter = gfxoutputdrv_list;
    return gfxoutputdrv_list_iter->drv;
}

gfxoutputdrv_t *gfxoutput_drivers_iter_next(void)
{
    if (gfxoutputdrv_list_iter)
        gfxoutputdrv_list_iter = gfxoutputdrv_list_iter->next;

    if (gfxoutputdrv_list_iter)
        return gfxoutputdrv_list_iter->drv;

    return NULL;
}

int gfxoutput_early_init(void)
{
    /* Initialize graphics output driver list.  */
    gfxoutputdrv_list
        = (gfxoutputdrv_list_t *)lib_malloc(sizeof(gfxoutputdrv_list_t));
    gfxoutputdrv_list->drv = NULL;
    gfxoutputdrv_list->next = NULL;

    gfxoutput_init_bmp();
#if defined(HAVE_GIF) || defined(WIN32)
    gfxoutput_init_gif();
#endif
    gfxoutput_init_iff();
#ifdef HAVE_JPEG
    gfxoutput_init_jpeg();
#endif
    gfxoutput_init_pcx();
#ifdef HAVE_PNG
    gfxoutput_init_png();
#endif
    gfxoutput_init_ppm();
#ifdef HAVE_FFMPEG
    gfxoutput_init_ffmpeg();
#endif

    return 0;
}

int gfxoutput_init(void)
{
    gfxoutput_log = log_open("Graphics Output");

    return 0;
}

void gfxoutput_shutdown(void)
{
    gfxoutputdrv_list_t *list, *next;

    list = gfxoutputdrv_list;

    while (list != NULL) {
        next = list->next;
        lib_free(list);
        list = next;
    }

#ifdef HAVE_FFMPEG
    ffmpegdrv_shutdown();
#endif
}

/*-----------------------------------------------------------------------*/

int gfxoutput_register(gfxoutputdrv_t *drv)
{
    gfxoutputdrv_list_t *current;

    current = gfxoutputdrv_list;

    /* Warp to end of list.  */
    while (current->next != NULL)
        current = current->next;

    /* Fill in entry.  */
    current->drv = drv;
    current->next = (gfxoutputdrv_list_t *)lib_malloc(
                    sizeof(gfxoutputdrv_list_t));
    current->next->drv = NULL;
    current->next->next = NULL;

    gfxoutputdrv_list_count++;

    return 0;
}

gfxoutputdrv_t *gfxoutput_get_driver(const char *drvname)
{
    gfxoutputdrv_list_t *current = gfxoutputdrv_list;

    while (current->next != NULL) {
       if (strcmp(drvname, current->drv->name) == 0
           || strcmp(drvname, current->drv->displayname) == 0)
           break;
       current = current->next;
    }

    /* Requested graphics output driver is not registered.  */
    if (current->next == NULL) {
        log_error(gfxoutput_log,
                  "Requested graphics output driver %s not found.",
                  drvname);
        return NULL;
    }
    return current->drv;
}

int  gfxoutput_resources_init()
{
#ifdef HAVE_FFMPEG
    if (ffmpegdrv_resources_init() < 0)
        return -1;
#endif

    return 0;
}

int gfxoutput_cmdline_options_init()
{
#ifdef HAVE_FFMPEG
    if (ffmpegdrv_cmdline_options_init() < 0)
        return -1;
#endif

    return 0;
}

int gfxoutput_convert_to_24bit(BYTE *buffer, BYTE *data, unsigned int width, unsigned int height, unsigned int depth, int color_config, palette_t *palette)
{
  int i;

  for (i = 0; i < width * height; i++)
  {
    switch (depth)
    {
      case 8:
        data[i*3]=palette->entries[buffer[i]].red;
        data[(i*3)+1]=palette->entries[buffer[i]].green;
        data[(i*3)+2]=palette->entries[buffer[i]].blue;
        break;
      case 16:
        switch (color_config)
        {
          case COLORCFG_15BIT_RGB555BE:
            data[ i*3   ]=(buffer[i*2]>>2)<<3;
            data[(i*3)+1]=(buffer[i*2]<<6)+((buffer[(i*2)+1]>>5)<<3);
            data[(i*3)+2]=buffer[(i*2)+1]<<3;
            break;
          case COLORCFG_15BIT_BGR555BE:
            data[ i*3   ]=buffer[(i*2)+1]<<3;
            data[(i*3)+1]=(buffer[i*2]<<6)+((buffer[(i*2)+1]>>5)<<3);
            data[(i*3)+2]=(buffer[i*2]>>2)<<3;
            break;
          case COLORCFG_15BIT_RGB555LE:
            data[ i*3   ]=(buffer[(i*2)+1]>>2)<<3;
            data[(i*3)+1]=(buffer[(i*2)+1]<<6)+((buffer[i*2]>>5)<<3);
            data[(i*3)+2]=buffer[i*2]<<3;
            break;
          case COLORCFG_15BIT_BGR555LE:
            data[ i*3   ]=buffer[i*2]<<3;
            data[(i*3)+1]=(buffer[(i*2)+1]<<6)+((buffer[i*2]>>5)<<3);
            data[(i*3)+2]=(buffer[(i*2)+1]>>2)<<3;
            break;
          case COLORCFG_16BIT_BGR565BE:
            data[ i*3   ]=buffer[(i*2)+1]<<3;
            data[(i*3)+1]=(buffer[i*2]<<5)+((buffer[(i*2)+1]>>5)<<2);
            data[(i*3)+2]=(buffer[i*2]>>3)<<3;
            break;
          case COLORCFG_16BIT_RGB565LE:
            data[ i*3   ]=(buffer[(i*2)+1]>>3)<<3;
            data[(i*3)+1]=(buffer[(i*2)+1]<<5)+((buffer[i*2]>>5)<<2);
            data[(i*3)+2]=buffer[i*2]<<3;
            break;
          case COLORCFG_16BIT_BGR565LE:
            data[ i*3   ]=buffer[i*2]<<3;
            data[(i*3)+1]=(buffer[(i*2)+1]<<5)+((buffer[i*2]>>5)<<2);
            data[(i*3)+2]=(buffer[i*2]>>3)<<3;
            break;
          case COLORCFG_16BIT_RGB565BE:
          default:
            data[ i*3   ]=(buffer[i*2]>>3)<<3;
            data[(i*3)+1]=(buffer[i*2]<<5)+((buffer[(i*2)+1]>>5)<<2);
            data[(i*3)+2]=buffer[(i*2)+1]<<3;
            break;
        }
        break;
      case 24:
        switch (color_config)
        {
          case COLORCFG_24BIT_BGR:
            data[ i*3   ]=buffer[(i*3)+2];
            data[(i*3)+1]=buffer[(i*3)+1];
            data[(i*3)+2]=buffer[ i*3   ];
            break;
          case COLORCFG_24BIT_RGB:
          default:
            data[ i*3   ]=buffer[ i*3   ];
            data[(i*3)+1]=buffer[(i*3)+1];
            data[(i*3)+2]=buffer[(i*3)+2];
            break;
        }
        break;
      case 32:
        switch (color_config)
        {
          case COLORCFG_32BIT_ARGB:
            data[ i*3   ]=buffer[(i*4)+1];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_ABGR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_BGRA:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_GARB:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_BRAG:
            data[ i*3   ]=buffer[(i*4)+1];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_GRAB:
            data[ i*3   ]=buffer[(i*4)+1];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_BARG:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_ARBG:
            data[ i*3   ]=buffer[(i*4)+1];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_AGRB:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_AGBR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_ABRG:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_RAGB:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_RABG:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_RGAB:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_RBAG:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_RBGA:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_GABR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_GRBA:
            data[ i*3   ]=buffer[(i*4)+1];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_GBAR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_GBRA:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_BAGR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_BRGA:
            data[ i*3   ]=buffer[(i*4)+1];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_BGAR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_RGBA:
          default:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[(i*4)+2];
            break;
        }
        break;
      default:
        return -1;
        break;
    }
  }
  return 0;
}

int gfxoutput_convert_to_32bit(BYTE *buffer, BYTE *data, unsigned int width, unsigned int height, unsigned int depth, int color_config, palette_t *palette)
{
  int i;

  for (i = 0; i < width * height; i++)
  {
    switch (depth)
    {
      case 8:
        data[ i*4   ]=palette->entries[buffer[i]].red;
        data[(i*4)+1]=palette->entries[buffer[i]].green;
        data[(i*4)+2]=palette->entries[buffer[i]].blue;
        data[(i*4)+3]=0;
        break;
      case 16:
        switch (color_config)
        {
          case COLORCFG_15BIT_RGB555BE:
            data[ i*4   ]=(buffer[i*2]>>2)<<3;
            data[(i*4)+1]=(buffer[i*2]<<6)+((buffer[(i*2)+1]>>5)<<3);
            data[(i*4)+2]=buffer[(i*2)+1]<<3;
            data[(i*4)+3]=0;
            break;
          case COLORCFG_15BIT_BGR555BE:
            data[ i*4   ]=buffer[(i*2)+1]<<3;
            data[(i*4)+1]=(buffer[i*2]<<6)+((buffer[(i*2)+1]>>5)<<3);
            data[(i*4)+2]=(buffer[i*2]>>2)<<3;
            data[(i*4)+3]=0;
            break;
          case COLORCFG_15BIT_RGB555LE:
            data[ i*4   ]=(buffer[(i*2)+1]>>2)<<3;
            data[(i*4)+1]=(buffer[(i*2)+1]<<6)+((buffer[i*2]>>5)<<3);
            data[(i*4)+2]=buffer[i*2]<<3;
            data[(i*4)+3]=0;
            break;
          case COLORCFG_15BIT_BGR555LE:
            data[ i*4   ]=buffer[i*2]<<3;
            data[(i*4)+1]=(buffer[(i*2)+1]<<6)+((buffer[i*2]>>5)<<3);
            data[(i*4)+2]=(buffer[(i*2)+1]>>2)<<3;
            data[(i*4)+3]=0;
            break;
          case COLORCFG_16BIT_BGR565BE:
            data[ i*4   ]=buffer[(i*2)+1]<<3;
            data[(i*4)+1]=(buffer[i*2]<<5)+((buffer[(i*2)+1]>>5)<<2);
            data[(i*4)+2]=(buffer[i*2]>>3)<<3;
            data[(i*4)+3]=0;
            break;
          case COLORCFG_16BIT_RGB565LE:
            data[ i*4   ]=(buffer[(i*2)+1]>>3)<<3;
            data[(i*4)+1]=(buffer[(i*2)+1]<<5)+((buffer[i*2]>>5)<<2);
            data[(i*4)+2]=buffer[i*2]<<3;
            data[(i*4)+3]=0;
            break;
          case COLORCFG_16BIT_BGR565LE:
            data[ i*4   ]=buffer[i*2]<<3;
            data[(i*4)+1]=(buffer[(i*2)+1]<<5)+((buffer[i*2]>>5)<<2);
            data[(i*4)+2]=(buffer[i*2]>>3)<<3;
            data[(i*4)+3]=0;
            break;
          case COLORCFG_16BIT_RGB565BE:
          default:
            data[ i*4   ]=(buffer[i*2]>>3)<<3;
            data[(i*4)+1]=(buffer[i*2]<<5)+((buffer[(i*2)+1]>>5)<<2);
            data[(i*4)+2]=buffer[(i*2)+1]<<3;
            data[(i*4)+3]=0;
            break;
        }
        break;
      case 24:
        switch (color_config)
        {
          case COLORCFG_24BIT_BGR:
            data[ i*4   ]=buffer[(i*3)+2];
            data[(i*4)+1]=buffer[(i*3)+1];
            data[(i*4)+2]=buffer[ i*3   ];
            data[(i*4)+3]=0;
            break;
          case COLORCFG_24BIT_RGB:
          default:
            data[ i*4   ]=buffer[ i*3   ];
            data[(i*4)+1]=buffer[(i*3)+1];
            data[(i*4)+2]=buffer[(i*3)+2];
            data[(i*4)+3]=0;
            break;
        }
        break;
      case 32:
        switch (color_config)
        {
          case COLORCFG_32BIT_ARGB:
            data[ i*4   ]=buffer[(i*4)+1];
            data[(i*4)+1]=buffer[(i*4)+2];
            data[(i*4)+2]=buffer[(i*4)+3];
            data[(i*4)+3]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_ABGR:
            data[ i*4   ]=buffer[(i*4)+3];
            data[(i*4)+1]=buffer[(i*4)+2];
            data[(i*4)+2]=buffer[(i*4)+1];
            data[(i*4)+3]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_BGRA:
            data[ i*4   ]=buffer[(i*4)+2];
            data[(i*4)+1]=buffer[(i*4)+1];
            data[(i*4)+2]=buffer[ i*4   ];
            data[(i*4)+3]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_GARB:
            data[ i*4   ]=buffer[(i*4)+2];
            data[(i*4)+1]=buffer[ i*4   ];
            data[(i*4)+2]=buffer[(i*4)+3];
            data[(i*4)+3]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_BRAG:
            data[ i*4   ]=buffer[(i*4)+1];
            data[(i*4)+1]=buffer[(i*4)+3];
            data[(i*4)+2]=buffer[ i*4   ];
            data[(i*4)+3]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_GRAB:
            data[ i*4   ]=buffer[(i*4)+1];
            data[(i*4)+1]=buffer[ i*4   ];
            data[(i*4)+2]=buffer[(i*4)+3];
            data[(i*4)+3]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_BARG:
            data[ i*4   ]=buffer[(i*4)+2];
            data[(i*4)+1]=buffer[(i*4)+3];
            data[(i*4)+1]=buffer[ i*4   ];
            data[(i*4)+3]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_ARBG:
            data[ i*4   ]=buffer[(i*4)+1];
            data[(i*4)+1]=buffer[(i*4)+3];
            data[(i*4)+2]=buffer[(i*4)+2];
            data[(i*4)+3]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_AGRB:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[(i*4)+3];
            data[(i*4)+3]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_AGBR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[(i*4)+2];
            data[(i*4)+3]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_ABRG:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[(i*4)+1];
            data[(i*4)+3]=buffer[ i*4   ];
            break;
          case COLORCFG_32BIT_RAGB:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[(i*4)+3];
            data[(i*4)+3]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_RABG:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[(i*4)+2];
            data[(i*4)+3]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_RGAB:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[(i*4)+3];
            data[(i*4)+3]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_RBAG:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+3];
            data[(i*3)+2]=buffer[(i*4)+1];
            data[(i*4)+3]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_RBGA:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[(i*4)+1];
            data[(i*4)+3]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_GABR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+2];
            data[(i*4)+3]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_GRBA:
            data[ i*3   ]=buffer[(i*4)+1];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+2];
            data[(i*4)+3]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_GBAR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+1];
            data[(i*4)+3]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_GBRA:
            data[ i*3   ]=buffer[(i*4)+2];
            data[(i*3)+1]=buffer[ i*4   ];
            data[(i*3)+2]=buffer[(i*4)+1];
            data[(i*4)+3]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_BAGR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[ i*4   ];
            data[(i*4)+3]=buffer[(i*4)+1];
            break;
          case COLORCFG_32BIT_BRGA:
            data[ i*3   ]=buffer[(i*4)+1];
            data[(i*3)+1]=buffer[(i*4)+2];
            data[(i*3)+2]=buffer[ i*4   ];
            data[(i*4)+3]=buffer[(i*4)+3];
            break;
          case COLORCFG_32BIT_BGAR:
            data[ i*3   ]=buffer[(i*4)+3];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[ i*4   ];
            data[(i*4)+3]=buffer[(i*4)+2];
            break;
          case COLORCFG_32BIT_RGBA:
          default:
            data[ i*3   ]=buffer[ i*4   ];
            data[(i*3)+1]=buffer[(i*4)+1];
            data[(i*3)+2]=buffer[(i*4)+2];
            data[(i*4)+3]=buffer[(i*4)+3];
            break;
        }
        break;
      default:
        return -1;
        break;
    }
  }
  return 0;
}
